@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # XEP-0145: Annotations
 
@@ -40,7 +30,7 @@ class Annotations(BaseModule):
 
     def request_annotations(self) -> None:
         self._nbxmpp('Annotations').request_annotations(
-            callback=self._annotations_received)
+            callback=self._annotations_received)  # type: ignore
 
     def _annotations_received(self, task: Task) -> None:
         try:
