@@ -36,7 +36,8 @@ from gajim.common.events import FileRequestError
 from gajim.common.file_props import FileProp
 from gajim.common.file_props import FilesProp
 from gajim.common.modules.base import BaseModule
-from gajim.common.socks5 import Socks5SenderClient
+
+# from gajim.common.socks5 import Socks5SenderClient
 from gajim.common.util.jid import get_full_jid_from_iq
 from gajim.common.util.jid import get_jid_from_iq
 from gajim.common.util.jid import parse_jid
@@ -112,9 +113,6 @@ class Bytestream(BaseModule):
             our_fjid = self._con.get_own_jid()
             testit = app.settings.get_account_setting(
                 self._account, 'test_ft_proxies_on_startup')
-            app.proxy65_manager.resolve(
-                info.jid, self._con.connection, str(our_fjid),
-                default=self._account, testit=testit)
             raise nbxmpp.NodeProcessed
 
     @staticmethod
@@ -461,44 +459,46 @@ class Bytestream(BaseModule):
     def _get_file_transfer_proxies_from_config(self,
                                                file_props: FileProp
                                                ) -> list[dict[str, Any]]:
-        configured_proxies = app.settings.get_account_setting(
-            self._account, 'file_transfer_proxies')
-        shall_use_proxies = app.settings.get_account_setting(
-            self._account, 'use_ft_proxies')
-        if shall_use_proxies:
-            proxyhost_dicts: list[dict[str, Any]] = []
-            proxies: list[str] = []
-            if configured_proxies:
-                proxies = [item.strip() for item in
-                           configured_proxies.split(',')]
-            default_proxy = app.proxy65_manager.get_default_for_name(
-                self._account)
-            if default_proxy:
-                # add/move default proxy at top of the others
-                if default_proxy in proxies:
-                    proxies.remove(default_proxy)
-                proxies.insert(0, default_proxy)
-
-            for proxy in proxies:
-                (host, _port, jid) = app.proxy65_manager.get_proxy(
-                    proxy, self._account)
-                if not host:
-                    continue
-                host_dict: dict[str, Any] = {
-                    'state': 0,
-                    'target': file_props.receiver,
-                    'id': file_props.sid,
-                    'sid': file_props.sid,
-                    'initiator': proxy,
-                    'host': host,
-                    'port': str(_port),
-                    'jid': jid
-                }
-                proxyhost_dicts.append(host_dict)
-            return proxyhost_dicts
-
         return []
 
+        # configured_proxies = app.settings.get_account_setting(
+        #     self._account, 'file_transfer_proxies')
+        # shall_use_proxies = app.settings.get_account_setting(
+        #     self._account, 'use_ft_proxies')
+        # if shall_use_proxies:
+        #     proxyhost_dicts: list[dict[str, Any]] = []
+        #     proxies: list[str] = []
+        #     if configured_proxies:
+        #         proxies = [item.strip() for item in
+        #                    configured_proxies.split(',')]
+        #     default_proxy = app.proxy65_manager.get_default_for_name(
+        #         self._account)
+        #     if default_proxy:
+        #         # add/move default proxy at top of the others
+        #         if default_proxy in proxies:
+        #             proxies.remove(default_proxy)
+        #         proxies.insert(0, default_proxy)
+
+        #     for proxy in proxies:
+        #         (host, _port, jid) = app.proxy65_manager.get_proxy(
+        #             proxy, self._account)
+        #         if not host:
+        #             continue
+        #         host_dict: dict[str, Any] = {
+        #             'state': 0,
+        #             'target': file_props.receiver,
+        #             'id': file_props.sid,
+        #             'sid': file_props.sid,
+        #             'initiator': proxy,
+        #             'host': host,
+        #             'port': str(_port),
+        #             'jid': jid
+        #         }
+        #         proxyhost_dicts.append(host_dict)
+        #     return proxyhost_dicts
+
+        # return []
+
     @staticmethod
     def _result_socks5_sid(sid: str, hash_id: str) -> None:
         '''
@@ -560,14 +560,13 @@ class Bytestream(BaseModule):
         self._con.connection.send(iq)
 
     def _on_bytestream_error(self,
-                             _con: types.xmppClient,
+                             _con: types.NBXMPPClient,
                              iq_obj: Iq,
                              _properties: IqProperties
                              ) -> None:
         id_ = iq_obj.getAttr('id')
         frm = get_full_jid_from_iq(iq_obj)
         query = iq_obj.getTag('query')
-        app.proxy65_manager.error_cb(frm, query)
         jid = get_jid_from_iq(iq_obj)
         id_ = id_[3:]
         file_props = FilesProp.getFilePropBySid(id_)
@@ -582,7 +581,7 @@ class Bytestream(BaseModule):
         raise nbxmpp.NodeProcessed
 
     def _on_bytestream_set(self,
-                           _con: types.xmppClient,
+                           _con: types.NBXMPPClient,
                            iq_obj: Iq,
                            _properties: IqProperties):
         target = iq_obj.getAttr('to')
@@ -643,7 +642,7 @@ class Bytestream(BaseModule):
         raise nbxmpp.NodeProcessed
 
     def _on_result(self,
-                   _con: types.xmppClient,
+                   _con: types.NBXMPPClient,
                    iq_obj: Iq,
                    _properties: IqProperties
                    ) -> None:
@@ -665,14 +664,13 @@ class Bytestream(BaseModule):
                     raise nbxmpp.NodeProcessed
 
     def _on_bytestream_result(self,
-                              _con: types.xmppClient,
+                              _con: types.NBXMPPClient,
                               iq_obj: Iq,
                               _properties: IqProperties
                               ) -> None:
         frm = self._ft_get_from(iq_obj)
         real_id = iq_obj.getAttr('id')
         query = iq_obj.getTag('query')
-        app.proxy65_manager.resolve_result(frm, query)
 
         try:
             streamhost = query.getTag('streamhost-used')
