@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 
 # Handles the jingle signalling protocol
@@ -31,27 +21,28 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import logging
 
 import nbxmpp
 from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import Iq
-from nbxmpp.structs import StanzaHandler
 from nbxmpp.structs import IqProperties
+from nbxmpp.structs import StanzaHandler
 
-from gajim.common import helpers
-from gajim.common import jingle_xtls
 from gajim.common import types
 from gajim.common.file_props import FileProp
-from gajim.common.modules.base import BaseModule
+from gajim.common.jingle_ft import JingleFileTransfer
+from gajim.common.jingle_rtp import JingleAudio
+from gajim.common.jingle_rtp import JingleVideo
 from gajim.common.jingle_session import JingleSession
 from gajim.common.jingle_session import JingleStates
-from gajim.common.jingle_ft import JingleFileTransfer
-from gajim.common.jingle_transport import JingleTransportSocks5
 from gajim.common.jingle_transport import JingleTransportIBB
-from gajim.common.jingle_rtp import JingleAudio, JingleVideo
+from gajim.common.jingle_transport import JingleTransportSocks5
+from gajim.common.modules.base import BaseModule
+from gajim.common.util.jid import get_full_jid_from_iq
+from gajim.common.util.jid import InvalidFormat
+from gajim.common.util.text import get_random_string
 
 logger = logging.getLogger('gajim.c.m.jingle')
 
@@ -71,14 +62,6 @@ class Jingle(BaseModule):
                           typ='set',
                           ns=Namespace.JINGLE,
                           callback=self._on_jingle_iq),
-            StanzaHandler(name='iq',
-                          typ='get',
-                          ns=Namespace.PUBKEY_PUBKEY,
-                          callback=self._on_pubkey_request),
-            StanzaHandler(name='iq',
-                          typ='result',
-                          ns=Namespace.PUBKEY_PUBKEY,
-                          callback=self._pubkey_result_received),
         ]
 
         # dictionary: sessionid => JingleSession object
@@ -100,28 +83,8 @@ class Jingle(BaseModule):
             self._sessions[sid].callbacks = []
             del self._sessions[sid]
 
-    def _on_pubkey_request(self,
-                           con: types.xmppClient,
-                           stanza: Iq,
-                           _properties: IqProperties
-                           ) -> None:
-        jid_from = helpers.get_full_jid_from_iq(stanza)
-        self._log.info('Pubkey request from %s', jid_from)
-        sid = stanza.getAttr('id')
-        jingle_xtls.send_cert(con, jid_from, sid)
-        raise nbxmpp.NodeProcessed
-
-    def _pubkey_result_received(self,
-                                con: types.xmppClient,
-                                stanza: Iq,
-                                _properties: IqProperties
-                                ) -> None:
-        jid_from = helpers.get_full_jid_from_iq(stanza)
-        self._log.info('Pubkey result from %s', jid_from)
-        jingle_xtls.handle_new_cert(con, stanza, jid_from)
-
     def _on_jingle_iq(self,
-                      _con: types.xmppClient,
+                      _con: types.NBXMPPClient,
                       stanza: Iq,
                       _properties: IqProperties
                       ) -> None:
@@ -136,8 +99,8 @@ class Jingle(BaseModule):
         '''
         # get data
         try:
-            jid = helpers.get_full_jid_from_iq(stanza)
-        except helpers.InvalidFormat:
+            jid = get_full_jid_from_iq(stanza)
+        except InvalidFormat:
             logger.warning('Invalid JID: %s, ignoring it', stanza.getFrom())
             return
         id_ = stanza.getID()
@@ -233,10 +196,9 @@ class Jingle(BaseModule):
                             jid: str,
                             file_props: FileProp,
                             request: bool = False
-                            ) -> Optional[str]:
+                            ) -> str | None:
         logger.info('start file transfer with file: %s', file_props)
         contact = self._con.get_module('Contacts').get_contact(jid)
-        use_security = contact.supports(Namespace.JINGLE_XTLS)
         jingle = JingleSession(self._con,
                                weinitiate=True,
                                jid=jid,
@@ -261,16 +223,15 @@ class Jingle(BaseModule):
         transfer = JingleFileTransfer(jingle,
                                       transport=transport,
                                       file_props=file_props,
-                                      use_security=use_security,
                                       senders=senders)
         file_props.transport_sid = transport.sid
         file_props.algo = self.__hash_support(contact)
-        jingle.add_content('file' + helpers.get_random_string(), transfer)
+        jingle.add_content('file' + get_random_string(), transfer)
         jingle.start_session()
         return transfer.transport.sid
 
     @staticmethod
-    def __hash_support(contact: types.ResourceContact) -> Optional[str]:
+    def __hash_support(contact: types.ResourceContact) -> str | None:
         if contact.supports(Namespace.HASHES_2):
             if contact.supports(Namespace.HASHES_BLAKE2B_512):
                 return 'blake2b-512'
@@ -287,9 +248,9 @@ class Jingle(BaseModule):
         return None
 
     def get_jingle_sessions(self,
-                            jid: Optional[str],
-                            sid: Optional[str] = None,
-                            media: Optional[str] = None
+                            jid: str | None,
+                            sid: str | None = None,
+                            media: str | None = None
                             ) -> list[JingleSession]:
         if sid:
             return [se for se in self._sessions.values() if se.sid == sid]
@@ -308,10 +269,10 @@ class Jingle(BaseModule):
 
     def get_file_info(self,
                       peerjid: str,
-                      hash_: Optional[str] = None,
-                      name: Optional[str] = None,
-                      _account: Optional[str] = None
-                      ) -> Optional[dict[str, Any]]:
+                      hash_: str | None = None,
+                      name: str | None = None,
+                      _account: str | None = None
+                      ) -> dict[str, Any] | None:
         if hash_:
             for file in self.files:  # DEBUG
                 # if f['hash'] == '1294809248109223':
@@ -325,9 +286,9 @@ class Jingle(BaseModule):
 
     def get_jingle_session(self,
                            jid: str,
-                           sid: Optional[str] = None,
-                           media: Optional[str] = None
-                           ) -> Optional[JingleSession]:
+                           sid: str | None = None,
+                           media: str | None = None
+                           ) -> JingleSession | None:
         if sid is not None:
             if sid in self._sessions:
                 return self._sessions[sid]
