@@ -1,56 +1,45 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
-from typing import Type
+from typing import cast
 
+import logging
 import os
 import sys
-import json
 import zipfile
-import logging
-from shutil import rmtree, move
-from dataclasses import dataclass
-from pathlib import Path
-from importlib.util import spec_from_file_location
+from collections.abc import Callable
 from importlib.util import module_from_spec
+from importlib.util import spec_from_file_location
+from pathlib import Path
+from shutil import move
+from shutil import rmtree
+from types import TracebackType
 
-from packaging.version import Version as V
-from packaging.requirements import Requirement
-
-import gajim
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import modules
 from gajim.common.client import Client
-from gajim.common.events import PluginAdded
-from gajim.common.events import PluginRemoved
-from gajim.common.i18n import _
 from gajim.common.exceptions import PluginsystemError
-from gajim.common.helpers import Singleton
-from gajim.plugins.plugins_i18n import _ as p_
+from gajim.common.i18n import _
+from gajim.common.modules import BaseModule
+from gajim.common.util.classes import Singleton
 
-from gajim.plugins.helpers import GajimPluginActivateException
-from gajim.plugins.helpers import is_shipped_plugin
-from gajim.plugins.gajimplugin import GajimPlugin
-from gajim.plugins.gajimplugin import GajimPluginException
+from .events import PluginAdded
+from .events import PluginRemoved
+from .gajimplugin import GajimPlugin
+from .gajimplugin import GajimPluginException
+from .helpers import GajimPluginActivateException
+from .helpers import is_shipped_plugin
+from .manifest import PluginManifest
 
 log = logging.getLogger('gajim.p.manager')
 
+RmErrorT = tuple[type[BaseException], BaseException, TracebackType]
+
 FIELDS = ('name',
           'short_name',
           'version',
@@ -58,86 +47,6 @@ FIELDS = ('name',
           'authors',
           'homepage')
 
-GAJIM_VERSION = V(gajim.__version__.split('+', 1)[0]).base_version
-
-
-@dataclass(frozen=True, eq=True)
-class PluginManifest:
-    name: str
-    short_name: str
-    description: str
-    authors: list[str]
-    homepage: str
-    config_dialog: bool
-    version: V
-    requirements: list[Requirement]
-    platforms: list[str]
-    path: Optional[Path] = None
-
-    def __hash__(self):
-        return hash(f'{self.short_name}_{self.version}')
-
-    @property
-    def is_usable(self) -> bool:
-        if not self._check_requirements():
-            return False
-
-        platform = sys.platform
-        if platform not in ('win32', 'darwin', 'linux'):
-            # sys.platform can return an unknown amount of unix/linux derivates
-            platform = 'others'
-
-        return platform in self.platforms
-
-    @property
-    def is_shipped(self) -> bool:
-        if self.path is None:
-            return False
-        return is_shipped_plugin(self.path)
-
-    def get_remote_url(self, repository_url: str) -> str:
-        filename = f'{self.short_name}_{self.version}.zip'
-        return f'{repository_url}/{self.short_name}/{filename}'
-
-    @classmethod
-    def from_path(cls, path: Path) -> PluginManifest:
-        manifest_path = path / 'plugin-manifest.json'
-        if not manifest_path.exists():
-            raise ValueError(f'Not a plugin path: {path}')
-
-        if manifest_path.is_dir():
-            raise ValueError(f'Not a plugin path: {path}')
-
-        with manifest_path.open(encoding='utf8') as f:
-            try:
-                manifest = json.load(f)
-            except Exception as error:
-                raise ValueError(f'Error while parsing manifest: '
-                                 f'{path}, {error}')
-        return cls.from_manifest_json(manifest, manifest_path)
-
-    def _check_requirements(self) -> bool:
-        for req in self.requirements:
-            if GAJIM_VERSION in req.specifier:
-                return True
-        return False
-
-    @classmethod
-    def from_manifest_json(cls,
-                           manifest: dict[str, Any],
-                           path: Optional[Path] = None) -> PluginManifest:
-        return cls(
-            name=manifest['name'],
-            short_name=manifest['short_name'],
-            description=p_(manifest['description']),
-            authors=manifest['authors'],
-            homepage=manifest['homepage'],
-            config_dialog=manifest['config_dialog'],
-            version=V(manifest['version']),
-            requirements=[Requirement(r) for r in manifest['requirements']],
-            platforms=manifest['platforms'],
-            path=path.parent if path is not None else path)
-
 
 class PluginManager(metaclass=Singleton):
     '''
@@ -179,7 +88,7 @@ class PluginManager(metaclass=Singleton):
     def update_plugins(self,
                        replace: bool = True,
                        activate: bool = False,
-                       plugin_name: Optional[str] = None
+                       plugin_name: str | None = None
                        ) -> list[str]:
         '''
         Move plugins from the downloaded folder to the user plugin folder
@@ -214,7 +123,7 @@ class PluginManager(metaclass=Singleton):
                     log.warning('Error while updating plugin')
                     continue
 
-                self.add_plugin(manifest, activate=True)
+                self._add_plugin(manifest, activate=True)
         return updated_plugins
 
     def init_plugins(self) -> None:
@@ -226,14 +135,14 @@ class PluginManager(metaclass=Singleton):
                 continue
 
             try:
-                self.activate_plugin(plugin)
+                self.activate_plugin(plugin, init=True)
             except GajimPluginActivateException as error:
                 plugin.activatable = False
                 plugin.available_text = str(error)
 
     def _load_plugin_module(self,
                             manifest: PluginManifest
-                            ) -> Optional[Type[GajimPlugin]]:
+                            ) -> type[GajimPlugin] | None:
 
         assert manifest.path is not None
         module_path = manifest.path / '__init__.py'
@@ -258,15 +167,15 @@ class PluginManager(metaclass=Singleton):
         for module_attr_name in dir(module):
             module_attr = getattr(module, module_attr_name)
             if issubclass(module_attr, GajimPlugin):
-                setattr(module_attr, 'manifest', manifest)
-                setattr(module_attr, '__path__', str(manifest.path))
+                module_attr.manifest = manifest
+                module_attr.__path__ = str(manifest.path)
                 return module_attr
         return None
 
-    def add_plugin(self,
-                   manifest: PluginManifest,
-                   activate: bool = False
-                   ) -> Optional[GajimPlugin]:
+    def _add_plugin(self,
+                    manifest: PluginManifest,
+                    activate: bool = False
+                    ) -> GajimPlugin | None:
 
         plugin_class = self._load_plugin_module(manifest)
         if plugin_class is None:
@@ -299,7 +208,7 @@ class PluginManager(metaclass=Singleton):
 
         return plugin_obj
 
-    def remove_plugin(self, plugin: GajimPlugin) -> None:
+    def _remove_plugin(self, plugin: GajimPlugin) -> None:
         '''
         removes the plugin from the plugin list and deletes all loaded modules
         from sys. This way we will have a fresh start when the plugin gets added
@@ -316,7 +225,7 @@ class PluginManager(metaclass=Singleton):
         # name to avoid name problems (removing module_abc if base_package is
         # module_ab)
         modules_to_remove = [module for module in sys.modules
-                             if module.startswith('{}.'.format(base_package))]
+                             if module.startswith(f'{base_package}.')]
         # remove the base_package itself
         if base_package in sys.modules:
             modules_to_remove.append(base_package)
@@ -324,13 +233,13 @@ class PluginManager(metaclass=Singleton):
         for module_to_remove in modules_to_remove:
             del sys.modules[module_to_remove]
 
-    def get_active_plugin(self, plugin_name: str) -> Optional[GajimPlugin]:
+    def get_active_plugin(self, plugin_name: str) -> GajimPlugin | None:
         for plugin in self.active_plugins:
             if plugin.manifest.short_name == plugin_name:
                 return plugin
         return None
 
-    def get_plugin(self, short_name: str) -> Optional[GajimPlugin]:
+    def get_plugin(self, short_name: str) -> GajimPlugin | None:
         for plugin in self.plugins:
             if plugin.manifest.short_name == short_name:
                 return plugin
@@ -493,35 +402,39 @@ class PluginManager(metaclass=Singleton):
             return
         for client in app.get_clients():
             for module in plugin.modules:
-                instance = client.get_module(module.name)
+                instance = cast(BaseModule, client.get_module(module.name))
                 modules.unregister_single_module(client, module.name)
 
                 for handler in instance.handlers:
                     client.connection.unregister_handler(handler)
 
-    def activate_plugin(self, plugin: GajimPlugin) -> None:
+    def activate_plugin(self, plugin: GajimPlugin, init: bool = False) -> None:
         '''
         :param plugin: plugin to be activated
         :type plugin: class object of `GajimPlugin` subclass
         '''
-        if not plugin.active and plugin.activatable:
+        if plugin.active or not plugin.activatable:
+            return
 
-            self._add_gui_extension_points_handlers_from_plugin(plugin)
-            self._add_encryption_name_from_plugin(plugin)
-            self._handle_all_gui_extension_points_with_plugin(plugin)
-            self._register_events_handlers_in_ged(plugin)
-            self._register_modules_with_handlers(plugin)
+        self._add_gui_extension_points_handlers_from_plugin(plugin)
+        self._add_encryption_name_from_plugin(plugin)
+        self._handle_all_gui_extension_points_with_plugin(plugin)
+        self._register_events_handlers_in_ged(plugin)
+        self._register_modules_with_handlers(plugin)
 
-            self.active_plugins.append(plugin)
-            try:
-                plugin.activate()
-            except GajimPluginException as e:
-                self.deactivate_plugin(plugin)
-                raise GajimPluginActivateException(str(e))
-            app.settings.set_plugin_setting(plugin.manifest.short_name,
-                                            'active',
-                                            True)
-            plugin.active = True
+        self.active_plugins.append(plugin)
+        try:
+            plugin.activate()
+        except GajimPluginException as e:
+            self.deactivate_plugin(plugin)
+            raise GajimPluginActivateException(str(e))
+        app.settings.set_plugin_setting(plugin.manifest.short_name,
+                                        'active',
+                                        True)
+        plugin.active = True
+
+        if not init:
+            app.commands.init()
 
     def deactivate_plugin(self, plugin: GajimPlugin) -> None:
         # remove GUI extension points handlers (provided by plug-in) from
@@ -560,6 +473,7 @@ class PluginManager(metaclass=Singleton):
                                         'active',
                                         False)
         plugin.active = False
+        app.commands.init()
 
     def _add_gui_extension_points_handlers_from_plugin(self,
                                                        plugin: GajimPlugin
@@ -609,11 +523,11 @@ class PluginManager(metaclass=Singleton):
                     client.connection.register_handler(handler)
 
     @staticmethod
-    def _load_manifest(plugin_path: Path) -> Optional[PluginManifest]:
+    def _load_manifest(plugin_path: Path) -> PluginManifest | None:
         try:
             return PluginManifest.from_path(plugin_path)
         except Exception as error:
-            log.warning('Unable to load manifest: %s', error)
+            log.debug('Unable to load manifest: %s', error)
             return None
 
     def _load_manifests(self) -> None:
@@ -628,7 +542,11 @@ class PluginManager(metaclass=Singleton):
                     continue
 
                 if not manifest.is_usable:
-                    log.info('Skipped plugin with not suitable requirements')
+                    log.info(
+                        'Skipped plugin with not suitable requirements: %s %s',
+                        manifest.short_name,
+                        manifest.requirements
+                    )
                     continue
 
                 same_plugin = manifests.get(manifest.short_name)
@@ -641,12 +559,12 @@ class PluginManager(metaclass=Singleton):
                 manifests[manifest.short_name] = manifest
 
         for manifest in manifests.values():
-            self.add_plugin(manifest)
+            self._add_plugin(manifest)
 
     def install_from_zip(self,
                          zip_filename: str,
                          overwrite: bool = False
-                         ) -> Optional[GajimPlugin]:
+                         ) -> GajimPlugin | None:
         '''
         Install plugin from zip and return plugin
         '''
@@ -655,7 +573,7 @@ class PluginManager(metaclass=Singleton):
         except zipfile.BadZipfile:
             # it is not zip file
             raise PluginsystemError(_('Archive corrupted'))
-        except IOError:
+        except OSError:
             raise PluginsystemError(_('Archive empty'))
 
         if zip_file.testzip():
@@ -665,8 +583,7 @@ class PluginManager(metaclass=Singleton):
         dirs: list[str] = []
         manifest = None
         for filename in zip_file.namelist():
-            if (filename.startswith('.') or filename.startswith('/') or
-                    ('/' not in filename)):
+            if (filename.startswith(('.', '/')) or ('/' not in filename)):
                 # members not safe
                 raise PluginsystemError(_('Archive is malformed'))
             if filename.endswith('/') and filename.find('/', 0, -1) < 0:
@@ -699,10 +616,14 @@ class PluginManager(metaclass=Singleton):
             rmtree(plugin_path)
             raise PluginsystemError(_('Installation failed'))
 
-        return self.add_plugin(manifest)
+        return self._add_plugin(manifest)
 
     def delete_plugin_files(self, plugin_path: Path) -> None:
-        def on_error(func, path, error):
+        def _on_error(func: Callable[..., Any],
+                      path: Path,
+                      error: RmErrorT
+                      ) -> None:
+
             if func is os.path.islink:
                 # if symlink
                 os.unlink(path)
@@ -710,7 +631,7 @@ class PluginManager(metaclass=Singleton):
             # access is denied or other
             raise PluginsystemError(str(error[1]))
 
-        rmtree(plugin_path, False, on_error)
+        rmtree(plugin_path, False, _on_error)  # type: ignore
 
     def uninstall_plugin(self, plugin: GajimPlugin) -> None:
         '''
@@ -719,7 +640,7 @@ class PluginManager(metaclass=Singleton):
         if not plugin:
             return
 
-        self.remove_plugin(plugin)
+        self._remove_plugin(plugin)
         self.delete_plugin_files(Path(plugin.__path__))
         if not is_shipped_plugin(Path(plugin.__path__)):
             path = configpaths.get('PLUGINS_BASE') / plugin.manifest.short_name
@@ -734,7 +655,7 @@ class PluginManager(metaclass=Singleton):
 
         app.ged.raise_event(PluginRemoved(manifest=plugin.manifest))
 
-    def get_plugin_by_path(self, plugin_dir: str) -> Optional[GajimPlugin]:
+    def get_plugin_by_path(self, plugin_dir: str) -> GajimPlugin | None:
         for plugin in self.plugins:
             if plugin.__path__ in plugin_dir:
                 return plugin
