@@ -6,17 +6,7 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -68,25 +58,8 @@ def is_rtl_text(text: str) -> bool:
     return False
 
 
-def ngettext(s_sing: str,
-             s_plural: str,
-             n: int,
-             replace_sing: str | None = None,
-             replace_plural: str | None = None) -> str:
-    '''
-    Use as:
-        i18n.ngettext(
-            'leave room %s', 'leave rooms %s', len(rooms), 'a', 'a, b, c')
-
-    In other words this is a hack to ngettext() to support %s %d etc..
-    '''
-    text = _trans.translation.ngettext(s_sing, s_plural, n)
-    if n == 1 and replace_sing is not None:
-        return text % replace_sing
-
-    if n > 1 and replace_plural is not None:
-        return text % replace_plural
-    return text
+def ngettext(s_sing: str, s_plural: str, n: int) -> str:
+    return _trans.translation.ngettext(s_sing, s_plural, n)
 
 
 class Translation:
@@ -121,7 +94,7 @@ class Translation:
         if sys.platform == 'darwin':
             return self._get_darwin_default_lang()
 
-        return locale.getdefaultlocale()[0] or 'en'
+        return locale.getlocale()[0] or 'en'
 
     def init(self) -> None:
         try:
