@@ -1,49 +1,44 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
-from typing import Iterator
-from typing import Optional
+from typing import Literal
 from typing import overload
-from typing import Union
 
+import operator
+from collections.abc import Iterator
 from datetime import datetime
-from datetime import timezone
+from datetime import UTC
 
-import cairo
+from gi.repository import Gdk
 from gi.repository import GLib
 from nbxmpp.const import Affiliation
 from nbxmpp.const import Chatstate
+from nbxmpp.const import PresenceShow
 from nbxmpp.const import Role
 from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 from nbxmpp.structs import DiscoInfo
+from nbxmpp.structs import HatData
 from nbxmpp.structs import LocationData
 from nbxmpp.structs import MucSubject
 from nbxmpp.structs import TuneData
 
 from gajim.common import app
 from gajim.common import types
+from gajim.common.client_modules import ClientModules
 from gajim.common.const import PresenceShowExt
 from gajim.common.const import SimpleClientState
-from gajim.common.helpers import chatstate_to_string
-from gajim.common.helpers import get_groupchat_name
 from gajim.common.helpers import Observable
 from gajim.common.modules.base import BaseModule
 from gajim.common.modules.util import LogAdapter
+from gajim.common.setting_values import AllContactSettings
+from gajim.common.setting_values import AllContactSettingsT
+from gajim.common.setting_values import AllGroupChatSettings
+from gajim.common.setting_values import AllGroupChatSettingsT
 from gajim.common.setting_values import BoolContactSettings
 from gajim.common.setting_values import BoolGroupChatSettings
 from gajim.common.setting_values import IntGroupChatSettings
@@ -53,6 +48,8 @@ from gajim.common.structs import MUCPresenceData
 from gajim.common.structs import PresenceData
 from gajim.common.structs import UNKNOWN_MUC_PRESENCE
 from gajim.common.structs import UNKNOWN_PRESENCE
+from gajim.common.util.muc import get_groupchat_name
+from gajim.common.util.user_strings import chatstate_to_string
 
 
 class ContactSettings:
@@ -65,14 +62,14 @@ class ContactSettings:
     @overload
     def get(self, setting: BoolContactSettings) -> bool: ...  # noqa: E704
 
-    def get(self, setting: Any) -> Any:
+    def get(self, setting: AllContactSettings) -> AllContactSettingsT:
         return app.settings.get_contact_setting(
             self._account, self._jid, setting)
 
     @overload
-    def set(self, setting: StringContactSettings, value: str) -> None: ...  # noqa: E501, E704
+    def set(self, setting: StringContactSettings, value: str | None) -> None: ...  # noqa: E501, E704
     @overload
-    def set(self, setting: BoolContactSettings, value: bool) -> None: ...  # noqa: E501, E704
+    def set(self, setting: BoolContactSettings, value: bool | None) -> None: ...  # noqa: E501, E704
 
     def set(self, setting: Any, value: Any) -> None:
         app.settings.set_contact_setting(
@@ -91,16 +88,16 @@ class GroupChatSettings:
     @overload
     def get(self, setting: IntGroupChatSettings) -> int: ...  # noqa: E704
 
-    def get(self, setting: Any) -> Any:
+    def get(self, setting: AllGroupChatSettings) -> AllGroupChatSettingsT:
         return app.settings.get_group_chat_setting(
             self._account, self._jid, setting)
 
     @overload
-    def set(self, setting: StringGroupChatSettings, value: str) -> None: ...  # noqa: E501, E704
+    def set(self, setting: StringGroupChatSettings, value: str | None) -> None: ...  # noqa: E501, E704
     @overload
-    def set(self, setting: BoolGroupChatSettings, value: bool) -> None: ...  # noqa: E501, E704
+    def set(self, setting: BoolGroupChatSettings, value: bool | None) -> None: ...  # noqa: E501, E704
     @overload
-    def set(self, setting: IntGroupChatSettings, value: int) -> None: ...  # noqa: E501, E704
+    def set(self, setting: IntGroupChatSettings, value: int | None) -> None: ...  # noqa: E501, E704
 
     def set(self, setting: Any, value: Any) -> None:
         app.settings.set_group_chat_setting(
@@ -111,7 +108,7 @@ class Contacts(BaseModule):
     def __init__(self, client: types.Client) -> None:
         BaseModule.__init__(self, client)
 
-        self._contacts: dict[JID, Union[BareContact, GroupchatContact]] = {}
+        self._contacts: dict[JID, BareContact | GroupchatContact] = {}
         self._con.connect_signal('state-changed', self._on_client_state_changed)
         self._con.connect_signal('resume-failed', self._on_client_resume_failed)
 
@@ -127,7 +124,7 @@ class Contacts(BaseModule):
         if state.is_disconnected:
             self._reset_presence()
 
-    def add_chat_contact(self, jid: Union[str, JID]) -> BareContact:
+    def add_chat_contact(self, jid: str | JID) -> BareContact:
         if isinstance(jid, str):
             jid = JID.from_string(jid)
 
@@ -143,31 +140,37 @@ class Contacts(BaseModule):
         self._contacts[jid] = contact
         return contact
 
-    def add_group_chat_contact(self, jid: Union[str, JID]) -> GroupchatContact:
+    def add_group_chat_contact(self, jid: str | JID) -> GroupchatContact:
         if isinstance(jid, str):
             jid = JID.from_string(jid)
 
         contact = self._contacts.get(jid)
         if contact is not None:
-            if not isinstance(contact, GroupchatContact):
-                raise ValueError(f'Trying to add GroupchatContact {jid}, '
-                                 f'but contact already exists as {contact} '
-                                 f'(in roster: {contact.is_in_roster})')
-            return contact
+            if isinstance(contact, GroupchatContact):
+                return contact
+
+            self._log.error(f'Trying to add GroupchatContact {jid}, '
+                            f'but contact already exists as {contact} '
+                            f'(in roster: {contact.is_in_roster})')
+            # Most likely a groupchat which has been added to the roster.
+            # Remove jid from roster (if we are online) and delete contact.
+            if self._client.state.is_available:
+                self._client.get_module('Roster').delete_item(jid)
+            self._contacts.pop(jid)
 
         contact = GroupchatContact(self._log, jid, self._account)
 
         self._contacts[jid] = contact
         return contact
 
-    def add_private_contact(self, jid: Union[str, JID]) -> GroupchatParticipant:
+    def add_private_contact(self, jid: str | JID) -> GroupchatParticipant:
         if isinstance(jid, str):
             jid = JID.from_string(jid)
 
         if jid.resource is None:
             raise ValueError(f'Trying to add a bare JID as private {jid}')
 
-        contact = self._contacts.get(jid.bare)
+        contact = self._contacts.get(jid.new_as_bare())
         if contact is None:
             group_chat_contact = self.add_group_chat_contact(jid.bare)
             return group_chat_contact.add_resource(jid.resource)
@@ -179,9 +182,8 @@ class Contacts(BaseModule):
         return contact.add_resource(jid.resource)
 
     def add_contact(self,
-                    jid: Union[str, JID],
-                    groupchat: bool = False) -> Union[BareContact,
-                                                      GroupchatContact]:
+                    jid: str | JID,
+                    groupchat: bool = False) -> BareContact | GroupchatContact:
         if isinstance(jid, str):
             jid = JID.from_string(jid)
 
@@ -198,7 +200,7 @@ class Contacts(BaseModule):
         return contact
 
     def get_contact(self,
-                    jid: Union[str, JID],
+                    jid: str | JID,
                     groupchat: bool = False
                     ) -> types.ContactT:
 
@@ -218,8 +220,9 @@ class Contacts(BaseModule):
         contact = contact.get_resource(resource)
         return contact
 
-    def get_bare_contact(self, jid: Union[str, JID]) -> Union[BareContact,
-                                                              GroupchatContact]:
+    def get_bare_contact(self,
+                         jid: str | JID
+                         ) -> BareContact | GroupchatContact:
         '''This method gives direct access to the contacts dict.
            This is helpful when performance is essential. In difference to
            get_contact() this method does not create contacts nor can it handle
@@ -230,10 +233,9 @@ class Contacts(BaseModule):
 
     def get_contacts_with_domain(self,
                                  domain: str
-                                 ) -> list[Union[BareContact,
-                                                 GroupchatContact]]:
+                                 ) -> list[BareContact | GroupchatContact]:
 
-        contacts: list[Union[BareContact, GroupchatContact]] = []
+        contacts: list[BareContact | GroupchatContact] = []
         for contact in self._contacts.values():
             if contact.jid.domain == domain:
                 contacts.append(contact)
@@ -251,7 +253,7 @@ class Contacts(BaseModule):
             contact.force_chatstate_update()
 
 
-class CommonContact(Observable):
+class CommonContact(Observable, ClientModules):
     def __init__(self,
                  logger: LogAdapter,
                  jid: JID,
@@ -259,8 +261,10 @@ class CommonContact(Observable):
                  ) -> None:
 
         Observable.__init__(self, logger)
+        ClientModules.__init__(self, account)
         self._jid = jid
         self._account = account
+        self._gateway_type: str | None = None
 
     def __hash__(self) -> int:
         return hash(f'{self._account}-{self._jid}')
@@ -272,9 +276,6 @@ class CommonContact(Observable):
         return (self._account == obj.account and
                 obj._jid == self._jid)
 
-    def _module(self, name: str) -> BaseModule:
-        return app.get_client(self._account).get_module(name)
-
     @property
     def jid(self) -> JID:
         return self._jid
@@ -283,15 +284,16 @@ class CommonContact(Observable):
     def account(self) -> str:
         return self._account
 
-    def _on_signal(self,
-                   _contact: Union[BareContact,
-                                   ResourceContact,
-                                   GroupchatContact,
-                                   GroupchatParticipant],
-                   signal_name: str,
-                   *args: Any,
-                   **kwargs: Any
-                   ) -> None:
+    def _on_signal(
+        self,
+        _contact: (BareContact |
+                   ResourceContact |
+                   GroupchatContact |
+                   GroupchatParticipant),
+        signal_name: str,
+        *args: Any,
+        **kwargs: Any
+    ) -> None:
         self.notify(signal_name, *args, **kwargs)
 
     def supports(self, requested_feature: str) -> bool:
@@ -329,7 +331,7 @@ class CommonContact(Observable):
         return self._jid
 
     @property
-    def chatstate(self) -> Optional[Chatstate]:
+    def chatstate(self) -> Chatstate | None:
         return None
 
     @property
@@ -343,7 +345,7 @@ class CommonContact(Observable):
             return False
 
         until = datetime.fromisoformat(mute_until)
-        is_muted = until > datetime.now(timezone.utc)
+        is_muted = until > datetime.now(UTC)
         if not is_muted:
             # Reset the setting to default
             GLib.idle_add(self.settings.set, 'mute_until', None)
@@ -352,6 +354,25 @@ class CommonContact(Observable):
     def __repr__(self) -> str:
         return f'{self.jid} ({self._account})'
 
+    def _get_transport_icon_name(self) -> str | None:
+        if self._gateway_type is not None:
+            return f'gateway-{self._gateway_type}'
+
+        domain_disco = app.storage.cache.get_last_disco_info(self._jid.domain)
+        if domain_disco is None:
+            return None
+
+        if gateway_type := domain_disco.gateway_type:
+            self._gateway_type = gateway_type
+            return f'gateway-{gateway_type}'
+
+    def update_gateway_type(self, gateway_type: str | None):
+        if gateway_type == self._gateway_type:
+            return
+
+        self._gateway_type = gateway_type
+        self.notify('avatar-update')
+
 
 class BareContact(CommonContact):
     def __init__(self, logger: LogAdapter, jid: JID, account: str) -> None:
@@ -363,21 +384,27 @@ class BareContact(CommonContact):
         self._avatar_sha = app.storage.cache.get_contact(
             account, jid, 'avatar')
 
+        self._presence = UNKNOWN_PRESENCE
+
     @property
     def is_self(self):
         own_jid = app.get_client(self._account).get_own_jid().new_as_bare()
         return own_jid == self.jid
 
     def supports(self, requested_feature: str) -> bool:
+        if not self._resources:
+            return super().supports(requested_feature)
         return any(resource.supports(requested_feature)
                    for resource in self.iter_resources())
 
+    @property
     def supports_audio(self) -> bool:
         if (self.supports(Namespace.JINGLE_ICE_UDP) and
                 app.is_installed('FARSTREAM')):
             return self.supports(Namespace.JINGLE_RTP_AUDIO)
         return False
 
+    @property
     def supports_video(self) -> bool:
         if (self.supports(Namespace.JINGLE_ICE_UDP) and
                 app.is_installed('FARSTREAM')):
@@ -400,12 +427,22 @@ class BareContact(CommonContact):
         contact.connect('caps-update', self._on_signal)
         return contact
 
+    def has_resources(self) -> bool:
+        return bool(self._resources)
+
     def get_resource(self, resource: str) -> ResourceContact:
         contact = self._resources.get(resource)
         if contact is None:
             contact = self.add_resource(resource)
         return contact
 
+    def get_active_resource(self) -> ResourceContact | None:
+        if not self._resources:
+            return None
+
+        return sorted(self._resources.values(),
+                      key=operator.attrgetter('show'))[-1]
+
     def get_resources(self) -> list[ResourceContact]:
         resources: list[ResourceContact] = []
         for contact in self._resources.values():
@@ -420,19 +457,42 @@ class BareContact(CommonContact):
 
     @property
     def is_available(self) -> bool:
-        # pylint: disable=R1729
-        return any([contact.is_available for contact
-                    in self._resources.values()])
+        if not self._resources:
+            return self._presence.available
+        return any(contact.is_available for contact
+                   in self._resources.values())
 
     @property
-    def show(self):
-        show_values = [contact.show for contact in self._resources.values()]
-        if not show_values:
-            return PresenceShowExt.OFFLINE
-        return max(show_values)
+    def show(self) -> PresenceShow | Literal[PresenceShowExt.OFFLINE]:
+        if not self._resources:
+            if not self._presence.available:
+                return PresenceShowExt.OFFLINE
+            return self._presence.show
+
+        res = self.get_active_resource()
+        assert res is not None
+        return res.show
 
     @property
-    def chatstate(self) -> Optional[Chatstate]:
+    def status(self) -> str:
+        if not self._resources:
+            return self._presence.status
+
+        res = self.get_active_resource()
+        assert res is not None
+        return res.status
+
+    @property
+    def idle_datetime(self) -> datetime | None:
+        if not self._resources:
+            return self._presence.idle_datetime
+
+        res = self.get_active_resource()
+        assert res is not None
+        return res.idle_datetime
+
+    @property
+    def chatstate(self) -> Chatstate | None:
         chatstates = {contact.chatstate for contact in self._resources.values()}
         chatstates.discard(None)
         if not chatstates:
@@ -445,7 +505,7 @@ class BareContact(CommonContact):
 
     @property
     def name(self) -> str:
-        item = self._module('Roster').get_item(self._jid)
+        item = self.get_module('Roster').get_item(self._jid)
         if item is not None and item.name:
             return item.name
 
@@ -460,18 +520,34 @@ class BareContact(CommonContact):
         assert self._jid.localpart is not None
         return self._jid.localpart
 
-    def get_tune(self) -> Optional[TuneData]:
-        return self._module('UserTune').get_contact_tune(self._jid)
+    @property
+    def original_name(self) -> str:
+        if self._jid.is_domain:
+            assert self._jid.domain is not None
+            original_name = self._jid.domain
+        else:
+            assert self._jid.localpart is not None
+            original_name = self._jid.localpart
 
-    def get_location(self) -> Optional[LocationData]:
-        return self._module('UserLocation').get_contact_location(self._jid)
+        nickname = app.storage.cache.get_contact(
+            self._account, self._jid, 'nickname')
+        if nickname is not None:
+            original_name = nickname
+
+        return original_name
+
+    def get_tune(self) -> TuneData | None:
+        return self.get_module('UserTune').get_contact_tune(self._jid)
+
+    def get_location(self) -> LocationData | None:
+        return self.get_module('UserLocation').get_contact_location(self._jid)
 
     @property
-    def avatar_sha(self) -> Optional[str]:
+    def avatar_sha(self) -> str | None:
         return app.storage.cache.get_contact(
             self._account, self._jid, 'avatar')
 
-    def set_avatar_sha(self, sha: str) -> None:
+    def set_avatar_sha(self, sha: str | None) -> None:
         app.storage.cache.set_contact(self._account, self._jid, 'avatar', sha)
 
     def get_avatar(self,
@@ -479,28 +555,14 @@ class BareContact(CommonContact):
                    scale: int,
                    add_show: bool = True,
                    default: bool = False,
-                   style: str = 'circle'):
+                   style: str = 'circle'
+                   ) -> Gdk.Texture:
 
         show = self.show.value if add_show else None
 
-        transport_icon = None
-        if self.is_gateway:
-            disco_info = app.storage.cache.get_last_disco_info(self._jid)
-            if disco_info is not None:
-                if disco_info.gateway_type == 'sms':
-                    transport_icon = 'gajim-agent-sms'
-                if disco_info.gateway_type == 'irc':
-                    transport_icon = 'gajim-agent-irc'
-        else:
-            for resource_contact in self.iter_resources():
-                if resource_contact.identity_type == 'sms':
-                    transport_icon = 'gajim-agent-sms'
-                    break
+        transport_icon = self._get_transport_icon_name()
 
-        if self.avatar_sha is not None:
-            transport_icon = None
-
-        return app.app.avatar_storage.get_surface(
+        return app.app.avatar_storage.get_texture(
             self,
             size,
             scale,
@@ -510,11 +572,13 @@ class BareContact(CommonContact):
             style=style)
 
     def update_presence(self, presence_data: PresenceData) -> None:
-        for contact in self._resources.values():
-            contact.update_presence(presence_data, notify=False)
+        self._presence = presence_data
+        if not presence_data.available:
+            for contact in self._resources.values():
+                contact.update_presence(presence_data, notify=False)
         self.notify('presence-update')
 
-    def update_avatar(self, sha: str) -> None:
+    def update_avatar(self, sha: str | None) -> None:
         if self._avatar_sha == sha:
             return
 
@@ -526,7 +590,7 @@ class BareContact(CommonContact):
 
     @property
     def is_in_roster(self) -> bool:
-        item = self._module('Roster').get_item(self._jid)
+        item = self.get_module('Roster').get_item(self._jid)
         return item is not None
 
     @property
@@ -537,22 +601,22 @@ class BareContact(CommonContact):
         return disco_info.is_gateway
 
     @property
-    def ask(self) -> Optional[str]:
-        item = self._module('Roster').get_item(self._jid)
+    def ask(self) -> str | None:
+        item = self.get_module('Roster').get_item(self._jid)
         if item is None:
             return None
         return item.ask
 
     @property
-    def subscription(self) -> Optional[str]:
-        item = self._module('Roster').get_item(self._jid)
+    def subscription(self) -> str | None:
+        item = self.get_module('Roster').get_item(self._jid)
         if item is None:
             return None
         return item.subscription
 
     @property
     def groups(self) -> set[str]:
-        item = self._module('Roster').get_item(self._jid)
+        item = self.get_module('Roster').get_item(self._jid)
         if item is None:
             return set()
         return item.groups
@@ -563,7 +627,7 @@ class BareContact(CommonContact):
 
     @property
     def is_blocked(self) -> bool:
-        return self._module('Blocking').is_blocked(self._jid)
+        return self.get_module('Blocking').is_blocked(self._jid)
 
     def set_blocked(self) -> None:
         self.update_presence(UNKNOWN_PRESENCE)
@@ -583,9 +647,18 @@ class BareContact(CommonContact):
         return True
 
     @property
-    def type_string(self) -> str:
+    def type_string(self) -> Literal['chat']:
         return 'chat'
 
+    @property
+    def is_bot(self) -> bool:
+        disco_info = app.storage.cache.get_last_disco_info(self._jid)
+
+        if disco_info is not None and disco_info.has_identity('client', 'bot'):
+            return True
+
+        return any(r.is_bot for r in self.iter_resources())
+
 
 class ResourceContact(CommonContact):
     def __init__(self, logger: LogAdapter, jid: JID, account: str) -> None:
@@ -604,7 +677,7 @@ class ResourceContact(CommonContact):
         return self._jid.resource
 
     @property
-    def identity_type(self) -> Optional[str]:
+    def identity_type(self) -> str | None:
         disco_info = app.storage.cache.get_last_disco_info(self._jid)
         if disco_info is None:
             return None
@@ -624,6 +697,12 @@ class ResourceContact(CommonContact):
         return any(identity.type == 'phone' for
                    identity in disco_info.identities)
 
+    @property
+    def is_bot(self):
+        disco_info = app.storage.cache.get_last_disco_info(self._jid)
+        return (disco_info is not None
+                and disco_info.has_identity('client', 'bot'))
+
     @property
     def is_available(self) -> bool:
         return self._presence.available
@@ -643,12 +722,12 @@ class ResourceContact(CommonContact):
         return self._presence.priority
 
     @property
-    def idle_time(self) -> Optional[float]:
-        return self._presence.idle_time
+    def idle_datetime(self) -> datetime | None:
+        return self._presence.idle_datetime
 
     @property
-    def chatstate(self) -> Chatstate:
-        return self._module('Chatstate').get_remote_chatstate(self._jid)
+    def chatstate(self) -> Chatstate | None:
+        return self.get_module('Chatstate').get_remote_chatstate(self._jid)
 
     def update_presence(self,
                         presence_data: PresenceData,
@@ -686,7 +765,7 @@ class GroupchatContact(CommonContact):
         return disco_info.is_irc
 
     @property
-    def muc_context(self) -> Optional[str]:
+    def muc_context(self) -> str | None:
         disco_info = self.get_disco()
         if disco_info is None:
             return None
@@ -727,6 +806,7 @@ class GroupchatContact(CommonContact):
         contact.connect('user-left', self._on_user_signal)
         contact.connect('user-affiliation-changed', self._on_user_signal)
         contact.connect('user-role-changed', self._on_user_signal)
+        contact.connect('user-hats-changed', self._on_user_signal)
         contact.connect('user-status-show-changed', self._on_user_signal)
         contact.connect('user-avatar-update', self._on_user_signal)
         return contact
@@ -748,21 +828,28 @@ class GroupchatContact(CommonContact):
         return get_groupchat_name(client, self._jid)
 
     @property
-    def avatar_sha(self) -> Optional[str]:
+    def original_name(self) -> str:
+        assert self._jid.localpart is not None
+        original_name = self._jid.localpart
+
+        disco_info = app.storage.cache.get_last_disco_info(self._jid)
+        if disco_info is not None:
+            if disco_info.muc_name:
+                original_name = disco_info.muc_name
+
+        return original_name
+
+    @property
+    def avatar_sha(self) -> str | None:
         return app.storage.cache.get_muc(self._account, self._jid, 'avatar')
 
-    def set_avatar_sha(self, sha: str) -> None:
+    def set_avatar_sha(self, sha: str | None) -> None:
         app.storage.cache.set_muc(self._account, self._jid, 'avatar', sha)
 
-    def get_avatar(self, size: int, scale: int) -> Optional[cairo.ImageSurface]:
-        transport_icon = None
-        disco_info = self.get_disco()
-        if disco_info is not None:
-            for identity in disco_info.identities:
-                if identity.type == 'irc':
-                    transport_icon = 'gajim-agent-irc'
-                break
-        return app.app.avatar_storage.get_muc_surface(
+    def get_avatar(self, size: int, scale: int) -> Gdk.Texture:
+        transport_icon = self._get_transport_icon_name()
+
+        return app.app.avatar_storage.get_muc_texture(
             self._account,
             self._jid,
             size,
@@ -783,50 +870,50 @@ class GroupchatContact(CommonContact):
         for contact in self._resources.values():
             contact.notify('chatstate-update')
 
-    def get_self(self) -> Optional[GroupchatParticipant]:
+    def get_self(self) -> GroupchatParticipant | None:
         nick = self.nickname
         if nick is None:
             return None
         return self.get_resource(nick)
 
     @property
-    def nickname(self) -> Optional[str]:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+    def nickname(self) -> str | None:
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return None
         return muc_data.nick
 
     @property
-    def occupant_jid(self) -> Optional[JID]:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+    def occupant_jid(self) -> JID | None:
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return None
         return muc_data.occupant_jid
 
     @property
-    def subject(self) -> Optional[MucSubject]:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+    def subject(self) -> MucSubject | None:
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return None
         return muc_data.subject
 
     @property
     def is_joined(self) -> bool:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return False
         return muc_data.state.is_joined
 
     @property
     def is_joining(self) -> bool:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return False
         return muc_data.state.is_joining
 
     @property
     def is_not_joined(self) -> bool:
-        muc_data = self._module('MUC').get_muc_data(self._jid)
+        muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return True
         return muc_data.state.is_not_joined
@@ -839,7 +926,7 @@ class GroupchatContact(CommonContact):
         client = app.get_client(self._account)
         return client.get_module('MUC').get_joined_users(self._jid)
 
-    def get_disco(self, max_age: int = 0) -> Optional[DiscoInfo]:
+    def get_disco(self, max_age: int = 0) -> DiscoInfo | None:
         return app.storage.cache.get_last_disco_info(self.jid, max_age=max_age)
 
     def can_notify(self) -> bool:
@@ -848,9 +935,15 @@ class GroupchatContact(CommonContact):
         return all_ or room
 
     @property
-    def type_string(self) -> str:
+    def type_string(self) -> Literal['groupchat']:
         return 'groupchat'
 
+    def has_composing_participants(self) -> bool:
+        return bool(self.get_module('Chatstate').get_composers(self._jid))
+
+    def get_composers(self) -> list['GroupchatParticipant']:
+        return self.get_module('Chatstate').get_composers(self._jid)
+
 
 class GroupchatParticipant(CommonContact):
     def __init__(self, logger: LogAdapter, jid: JID, account: str) -> None:
@@ -888,10 +981,12 @@ class GroupchatParticipant(CommonContact):
 
     @property
     def room(self) -> GroupchatContact:
-        return self._module('Contacts').get_bare_contact(self.jid.bare)
+        contact = self.get_module('Contacts').get_bare_contact(self.jid.bare)
+        assert isinstance(contact, GroupchatContact)
+        return contact
 
     @property
-    def muc_context(self) -> Optional[str]:
+    def muc_context(self) -> str | None:
         return self.room.muc_context
 
     @property
@@ -916,8 +1011,8 @@ class GroupchatParticipant(CommonContact):
         return self._presence.status
 
     @property
-    def idle_time(self) -> Optional[float]:
-        return self._presence.idle_time
+    def idle_datetime(self) -> datetime | None:
+        return self._presence.idle_datetime
 
     @property
     def name(self) -> str:
@@ -925,15 +1020,17 @@ class GroupchatParticipant(CommonContact):
         return self._jid.resource
 
     @property
-    def real_jid(self) -> Optional[JID]:
+    def real_jid(self) -> JID | None:
         return self._presence.real_jid
 
-    def get_real_contact(self) -> Optional[BareContact]:
+    def get_real_contact(self) -> BareContact | None:
         jid = self._presence.real_jid
         if jid is None:
             return None
-        return self._client.get_module('Contacts').get_contact(
+        contact = self._client.get_module('Contacts').get_contact(
             jid.new_as_bare())
+        assert isinstance(contact, BareContact)
+        return contact
 
     @property
     def affiliation(self) -> Affiliation:
@@ -944,22 +1041,23 @@ class GroupchatParticipant(CommonContact):
         return self._presence.role
 
     @property
-    def chatstate(self) -> Chatstate:
-        return self._module('Chatstate').get_remote_chatstate(self._jid)
+    def chatstate(self) -> Chatstate | None:
+        return self.get_module('Chatstate').get_remote_chatstate(self._jid)
 
     @property
-    def avatar_sha(self) -> str:
-        return self._client.get_module('VCardAvatars').get_avatar_sha(self._jid)
+    def avatar_sha(self) -> str | None:
+        return self._client.get_module('VCardAvatars').get_avatar_sha(
+            self._jid)
 
     def get_avatar(self,
                    size: int,
                    scale: int,
                    add_show: bool = True,
                    style: str = 'circle'
-                   ) -> cairo.ImageSurface:
+                   ) -> Gdk.Texture:
 
         show = self.show.value if add_show else None
-        return app.app.avatar_storage.get_surface(
+        return app.app.avatar_storage.get_texture(
             self, size, scale, show, style=style)
 
     def update_presence(self, presence: MUCPresenceData) -> None:
@@ -970,20 +1068,25 @@ class GroupchatParticipant(CommonContact):
         self.notify('user-avatar-update')
 
     @property
-    def type_string(self) -> str:
+    def type_string(self) -> Literal['pm']:
         return 'pm'
 
+    @property
+    def occupant_id(self) -> str | None:
+        return self._presence.occupant_id
 
-def can_add_to_roster(contact: Union[BareContact,
-                                     GroupchatContact,
-                                     GroupchatParticipant]) -> bool:
+    @property
+    def is_self(self) -> bool:
+        data = self.get_module('MUC').get_muc_data(self.room.jid)
+        assert data is not None
+        return data.nick == self.name
 
-    if isinstance(contact, GroupchatContact):
-        return False
+    @property
+    def hats(self) -> HatData | None:
+        return self._presence.hats
 
-    if isinstance(contact, GroupchatParticipant):
-        return contact.real_jid is not None
 
+def can_add_to_roster(contact: BareContact) -> bool:
     if contact.is_self:
         return False
     return not contact.is_in_roster
