@@ -9,24 +9,29 @@
 from __future__ import annotations
 
 from typing import Any
+from typing import cast
+from typing import Literal
 
 import binascii
 import threading
 from collections.abc import Callable
+from collections.abc import Generator
 from pathlib import Path
 
+import nbxmpp
 from gi.repository import GLib
 from nbxmpp.const import Affiliation
 from nbxmpp.const import PresenceType
+from nbxmpp.errors import is_error
 from nbxmpp.errors import StanzaError
 from nbxmpp.modules.omemo import create_omemo_message
 from nbxmpp.modules.omemo import get_key_transport_message
-from nbxmpp.modules.util import is_error
 from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 from nbxmpp.protocol import Message
 from nbxmpp.protocol import NodeProcessed
 from nbxmpp.protocol import Presence
+from nbxmpp.structs import AffiliationResult
 from nbxmpp.structs import EncryptionData
 from nbxmpp.structs import MessageProperties
 from nbxmpp.structs import OMEMOMessage
@@ -58,7 +63,9 @@ from gajim.common.events import MucDiscoUpdate
 from gajim.common.events import SignedIn
 from gajim.common.i18n import _
 from gajim.common.modules.base import BaseModule
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.modules.httpupload import HTTPFileTransfer
 from gajim.common.modules.util import as_task
 from gajim.common.modules.util import event_node
@@ -190,7 +197,7 @@ class OMEMO(BaseModule):
 
     def check_send_preconditions(self, contact: types.ChatContactT) -> bool:
         jid = str(contact.jid)
-        if contact.is_groupchat:
+        if isinstance(contact, GroupchatContact):
             if not self._groupchat_pre_conditions_satisfied(contact):
                 return False
 
@@ -326,7 +333,7 @@ class OMEMO(BaseModule):
         GLib.idle_add(callback, transfer)
 
     def _send_key_transport_message(self,
-                                    typ: str,
+                                    typ: Literal['chat', 'groupchat'],
                                     jid: str,
                                     devices: list[int]
                                     ) -> None:
@@ -342,15 +349,15 @@ class OMEMO(BaseModule):
         self._client.send_stanza(transport_message)
 
     def _message_received(self,
-                          _client: types.xmppClient,
+                          _client: types.NBXMPPClient,
                           stanza: Message,
                           properties: MessageProperties
                           ) -> None:
 
-        if not properties.is_omemo:
+        if properties.omemo is None:
             return
 
-        if properties.is_carbon_message and properties.carbon.is_sent:
+        if properties.carbon is not None and properties.carbon.is_sent:
             from_jid = self._own_jid
 
         elif properties.is_mam_message:
@@ -412,6 +419,7 @@ class OMEMO(BaseModule):
                              properties: MessageProperties
                              ) -> str | None:
 
+        assert properties.jid is not None
         resource = properties.jid.resource
         if properties.muc_ofrom is not None:
             # History Message from MUC
@@ -419,6 +427,7 @@ class OMEMO(BaseModule):
 
         contact = self._client.get_module('Contacts').get_contact(
             properties.jid)
+        assert isinstance(contact, GroupchatParticipant)
         if contact.real_jid is not None:
             return contact.real_jid.bare
 
@@ -441,7 +450,7 @@ class OMEMO(BaseModule):
         return properties.from_.bare
 
     def _on_muc_user_presence(self,
-                              _client: types.xmppClient,
+                              _client: types.NBXMPPClient,
                               _stanza: Presence,
                               properties: PresenceProperties
                               ) -> None:
@@ -452,6 +461,7 @@ class OMEMO(BaseModule):
         if properties.is_muc_destroyed:
             return
 
+        assert properties.jid is not None
         room = properties.jid.bare
 
         if properties.muc_user is None or properties.muc_user.jid is None:
@@ -487,6 +497,7 @@ class OMEMO(BaseModule):
             self._log.info('Affiliation request failed: %s', error)
             return
 
+        assert isinstance(result, AffiliationResult)
         for user_jid in result.users:
             jid = str(user_jid)
             self.backend.add_group_member(room_jid, jid)
@@ -504,6 +515,7 @@ class OMEMO(BaseModule):
             return False
 
         contact = self._client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
         return contact.subscription == 'both'
 
     def _check_if_omemo_capable(self, jid: str) -> None:
@@ -539,7 +551,7 @@ class OMEMO(BaseModule):
         self.request_bundle(jid, device_id)
 
     @as_task
-    def request_bundle(self, jid: str, device_id: int):
+    def request_bundle(self, jid: str, device_id: int) -> Generator[Any, Any]:
         _task = yield  # noqa: F841
 
         self._log.info('Request device bundle %s %s', device_id, jid)
@@ -585,7 +597,7 @@ class OMEMO(BaseModule):
         self.request_devicelist(jid)
 
     @as_task
-    def request_devicelist(self, jid: str | None = None):
+    def request_devicelist(self, jid: str | None = None) -> Generator[Any, Any]:
         _task = yield  # noqa: F841
 
         if jid is None:
@@ -598,19 +610,20 @@ class OMEMO(BaseModule):
             self._log.info('Devicelist request failed: %s %s', jid, devicelist)
             devicelist = []
 
-        self._process_devicelist_update(jid, devicelist)
+        self._process_devicelist_update(jid, cast(list[int], devicelist))
 
     @event_node(Namespace.OMEMO_TEMP_DL)
     def _devicelist_notification_received(self,
-                                          _client: types.xmppClient,
+                                          _client: types.NBXMPPClient,
                                           _stanza: Message,
                                           properties: MessageProperties
                                           ) -> None:
 
+        assert properties.pubsub_event is not None
         if properties.pubsub_event.retracted:
             return
 
-        devicelist = properties.pubsub_event.data or []
+        devicelist = cast(list[int], properties.pubsub_event.data or [])
 
         self._process_devicelist_update(str(properties.jid), devicelist)
 
@@ -635,7 +648,7 @@ class OMEMO(BaseModule):
 
         self._request_bundles_for_new_devices(jid)
 
-    def _debug_print_stanza(self, stanza: Any) -> None:
+    def _debug_print_stanza(self, stanza: nbxmpp.Node) -> None:
         stanzastr = '\n' + stanza.__str__(fancy=True)
         stanzastr = stanzastr[0:-1]
         self._log.debug(stanzastr)
