@@ -8,17 +8,7 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -29,14 +19,14 @@ import logging
 import keyring
 
 from gajim.common import app
-from gajim.common.helpers import package_version
+from gajim.common.util.version import package_version
 
 __all__ = [
+    'delete_password',
+    'get_password',
     'init',
     'is_keyring_available',
-    'get_password',
-    'save_password',
-    'delete_password'
+    'save_password'
 ]
 
 log = logging.getLogger('gajim.c.passwords')
@@ -94,10 +84,10 @@ class SecretPasswordStorage:
         if not is_keyring_available():
             return
 
-        log.info('Request password from keyring')
-
         account_jid = app.get_jid_from_account(account_name)
 
+        log.info('Request password from keyring for %s', account_jid)
+
         try:
             # For security reasons remove clear-text password
             ConfigPasswordStorage.delete_password(account_name)
@@ -132,14 +122,19 @@ class SecretPasswordStorage:
 
         account_jid = app.get_jid_from_account(account_name)
 
+        keyring_errors = (
+            keyring.errors.InitError,
+            keyring.errors.KeyringLocked,
+            keyring.errors.PasswordDeleteError,
+        )
         try:
             _interface.backend.delete_password('gajim', account_name)
-        except keyring.errors.PasswordDeleteError:
+        except keyring_errors:
             pass
 
         try:
             return _interface.backend.delete_password('gajim', account_jid)
-        except keyring.errors.PasswordDeleteError as error:
+        except keyring_errors as error:
             log.warning('Removing password failed: %s', error)
         except Exception:
             log.exception('Removing password failed')
