@@ -27,11 +27,13 @@ from gajim.common.events import JingleErrorReceived
 from gajim.common.events import JingleFtCancelledReceived
 from gajim.common.file_props import FileProp
 from gajim.common.file_props import FilesProp
+from gajim.common.ged import EventHelper
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.storage.archive import models as mod
 from gajim.common.storage.archive.const import ChatDirection
 from gajim.common.util.datetime import utc_now
+from gajim.common.util.text import format_eta
 from gajim.common.util.uri import open_file
 from gajim.common.util.uri import show_in_folder
 
@@ -39,23 +41,24 @@ from gajim.gtk.builder import get_builder
 from gajim.gtk.conversation.rows.base import BaseRow
 from gajim.gtk.conversation.rows.widgets import DateTimeLabel
 from gajim.gtk.conversation.rows.widgets import NicknameLabel
-from gajim.gtk.util import format_eta
 
 TransferEventT = FileRequestReceivedEvent | FileRequestSent
 
-log = logging.getLogger('gajim.gtk.conversation.rows.file_transfer_jingle')
+log = logging.getLogger("gajim.gtk.conversation.rows.file_transfer_jingle")
 
 
-class FileTransferJingleRow(BaseRow):
-    def __init__(self,
-                 account: str,
-                 contact: BareContact,
-                 event: TransferEventT | None = None,
-                 message: mod.Message | None = None
-                 ) -> None:
+class FileTransferJingleRow(BaseRow, EventHelper):
+    def __init__(
+        self,
+        account: str,
+        contact: BareContact,
+        event: TransferEventT | None = None,
+        message: mod.Message | None = None,
+    ) -> None:
         BaseRow.__init__(self, account)
+        EventHelper.__init__(self)
 
-        self.type = 'file-transfer'
+        self.type = "file-transfer"
 
         if message is not None:
             timestamp = message.timestamp
@@ -74,14 +77,14 @@ class FileTransferJingleRow(BaseRow):
                 if isinstance(source, mod.JingleFT):
                     self._file_props = FilesProp.getFilePropBySid(source.sid)
                     if self._file_props is None:
-                        log.debug('File prop not found for SID: %s', source.sid)
+                        log.debug("File prop not found for SID: %s", source.sid)
             self.pk = message.pk
         else:
             assert event is not None
             self._file_props = event.file_props
         self._start_time = 0
 
-        if app.settings.get('use_kib_mib'):
+        if app.settings.get("use_kib_mib"):
             self._units = GLib.FormatSizeFlags.IEC_UNITS
         else:
             self._units = GLib.FormatSizeFlags.DEFAULT
@@ -96,15 +99,17 @@ class FileTransferJingleRow(BaseRow):
                 contact = self._contact
                 is_self = True
             else:
-                bare_contact = self._client.get_module('Contacts').get_contact(
-                    self._client.get_own_jid().bare)
+                bare_contact = self._client.get_module("Contacts").get_contact(
+                    self._client.get_own_jid().bare
+                )
                 assert isinstance(bare_contact, BareContact)
                 contact = bare_contact
                 is_self = False
         else:
             if isinstance(event, FileRequestSent):
-                bare_contact = self._client.get_module('Contacts').get_contact(
-                    self._client.get_own_jid().bare)
+                bare_contact = self._client.get_module("Contacts").get_contact(
+                    self._client.get_own_jid().bare
+                )
                 assert isinstance(bare_contact, BareContact)
                 contact = bare_contact
                 is_self = False
@@ -115,8 +120,8 @@ class FileTransferJingleRow(BaseRow):
         scale = self.get_scale_factor()
         avatar = contact.get_avatar(AvatarSize.ROSTER, scale, add_show=False)
         assert not isinstance(avatar, GdkPixbuf.Pixbuf)
-        avatar_image = Gtk.Image.new_from_surface(avatar)
-        avatar_placeholder.add(avatar_image)
+        avatar_image = Gtk.Image.new_from_paintable(avatar)
+        avatar_placeholder.append(avatar_image)
 
         name_widget = NicknameLabel(contact.name, is_self)
         name_widget.set_halign(Gtk.Align.START)
@@ -128,17 +133,28 @@ class FileTransferJingleRow(BaseRow):
 
         meta_box = Gtk.Box()
         meta_box.set_spacing(6)
-        meta_box.add(name_widget)
-        meta_box.add(timestamp_widget)
+        meta_box.append(name_widget)
+        meta_box.append(timestamp_widget)
         self.grid.attach(meta_box, 1, 0, 1, 1)
 
-        self._ui = get_builder('file_transfer_jingle.ui')
+        self._ui = get_builder("file_transfer_jingle.ui")
         self.grid.attach(self._ui.transfer_box, 1, 1, 1, 1)
         self._ui.transfer_box.set_halign(Gtk.Align.START)
 
-        self._ui.connect_signals(self)
-
-        self.show_all()
+        self._connect(
+            self._ui.accept_file_request, "clicked", self._on_accept_file_request
+        )
+        self._connect(
+            self._ui.reject_file_request, "clicked", self._on_reject_file_request
+        )
+        self._connect(self._ui.open_folder, "clicked", self._on_open_folder)
+        self._connect(self._ui.open_file, "clicked", self._on_open_file)
+        self._connect(self._ui.error_show_transfers, "clicked", self._on_show_transfers)
+        self._connect(self._ui.retry_bad_hash, "clicked", self._on_bad_hash_retry)
+        self._connect(
+            self._ui.rejected_show_transfers, "clicked", self._on_show_transfers
+        )
+        self._connect(self._ui.cancel_transfer, "clicked", self._on_cancel_transfer)
 
         if message is not None:
             self._reconstruct_transfer()
@@ -149,29 +165,29 @@ class FileTransferJingleRow(BaseRow):
         if self._file_props is None:
             return
 
-        app.ged.register_event_handler(
-            'file-completed', ged.GUI1, self.process_event)
-        app.ged.register_event_handler(
-            'file-hash-error', ged.GUI1, self.process_event)
-        app.ged.register_event_handler(
-            'file-send-error', ged.GUI1, self.process_event)
-        app.ged.register_event_handler(
-            'file-request-error', ged.GUI1, self.process_event)
-        app.ged.register_event_handler(
-            'file-progress', ged.GUI1, self.process_event)
-        app.ged.register_event_handler(
-            'file-error', ged.GUI1, self.process_event)
-        app.ged.register_event_handler(
-            'jingle-error-received', ged.GUI1, self.process_event)
-        app.ged.register_event_handler(
-            'jingle-ft-cancelled-received', ged.GUI1, self.process_event)
+        self.register_events(
+            [
+                ("file-completed", ged.GUI1, self.process_event),
+                ("file-hash-error", ged.GUI1, self.process_event),
+                ("file-send-error", ged.GUI1, self.process_event),
+                ("file-request-error", ged.GUI1, self.process_event),
+                ("file-progress", ged.GUI1, self.process_event),
+                ("file-error", ged.GUI1, self.process_event),
+                ("jingle-error-received", ged.GUI1, self.process_event),
+                ("jingle-ft-cancelled-received", ged.GUI1, self.process_event),
+            ]
+        )
+
+    def do_unroot(self) -> None:
+        self.unregister_events()
+        BaseRow.do_unroot(self)
 
     def _reconstruct_transfer(self) -> None:
         self._show_file_infos()
         if self._file_props is None:
-            self._ui.transfer_action.set_text(_('File Transfer'))
-            self._ui.error_label.set_text(_('No info available'))
-            self._ui.action_stack.set_visible_child_name('error')
+            self._ui.transfer_action.set_text(_("File Transfer"))
+            self._ui.error_label.set_text(_("No info available"))
+            self._ui.action_stack.set_visible_child_name("error")
             return
 
         if self._file_props.completed:
@@ -179,35 +195,35 @@ class FileTransferJingleRow(BaseRow):
             return
 
         if self._file_props.stopped:
-            self._ui.action_stack.set_visible_child_name('error')
-            self._ui.transfer_action.set_text(_('File Transfer Stopped'))
-            self._ui.error_label.set_text('')
+            self._ui.action_stack.set_visible_child_name("error")
+            self._ui.transfer_action.set_text(_("File Transfer Stopped"))
+            self._ui.error_label.set_text("")
             return
 
         if self._file_props.error is not None:
             self._show_error(self._file_props)
             return
 
-        self._ui.transfer_action.set_text(_('File Offered…'))
+        self._ui.transfer_action.set_text(_("File Offered…"))
 
     def _display_transfer_info(self, event_name: str) -> None:
-        if event_name == 'file-request-sent':
-            self._ui.action_stack.set_visible_child_name('progress')
-            self._ui.progress_label.set_text(_('Waiting…'))
+        if event_name == "file-request-sent":
+            self._ui.action_stack.set_visible_child_name("progress")
+            self._ui.progress_label.set_text(_("Waiting…"))
 
-        self._ui.transfer_action.set_text(_('File Offered…'))
+        self._ui.transfer_action.set_text(_("File Offered…"))
         self._show_file_infos()
 
     def _show_file_infos(self) -> None:
         if self._file_props is None:
-            self._ui.file_name.hide()
-            self._ui.file_description.hide()
-            self._ui.file_size.hide()
+            self._ui.file_name.set_visible(False)
+            self._ui.file_description.set_visible(False)
+            self._ui.file_size.set_visible(False)
             return
 
         file_name = GLib.markup_escape_text(str(self._file_props.name))
         if self._file_props.mime_type:
-            file_name = f'{file_name} ({self._file_props.mime_type})'
+            file_name = f"{file_name} ({self._file_props.mime_type})"
         self._ui.file_name.set_text(file_name)
         self._ui.file_name.set_tooltip_text(file_name)
 
@@ -216,11 +232,12 @@ class FileTransferJingleRow(BaseRow):
             self._ui.file_description.set_text(desc)
             self._ui.file_description.set_tooltip_text(desc)
         else:
-            self._ui.file_description.hide()
+            self._ui.file_description.set_visible(False)
 
         assert self._file_props.size is not None
         self._ui.file_size.set_text(
-            GLib.format_size_full(self._file_props.size, self._units))
+            GLib.format_size_full(self._file_props.size, self._units)
+        )
 
     def process_event(self, event: TransferEventT) -> None:
         assert self._file_props is not None
@@ -228,20 +245,20 @@ class FileTransferJingleRow(BaseRow):
         if isinstance(event, JingleErrorReceived):
             if event.sid != self._file_props.sid:
                 return
-            self._ui.action_stack.set_visible_child_name('error')
-            self._ui.transfer_action.set_text(_('File Transfer Cancelled'))
+            self._ui.action_stack.set_visible_child_name("error")
+            self._ui.transfer_action.set_text(_("File Transfer Cancelled"))
             self._ui.error_label.set_text(event.reason)
             return
 
         if isinstance(event, JingleFtCancelledReceived):
             if event.sid != self._file_props.sid:
                 return
-            self._ui.action_stack.set_visible_child_name('error')
-            self._ui.transfer_action.set_text(_('File Transfer Cancelled'))
+            self._ui.action_stack.set_visible_child_name("error")
+            self._ui.transfer_action.set_text(_("File Transfer Cancelled"))
             self._ui.error_label.set_text(
-                _('%(name)s cancelled the transfer (%(reason)s)') % {
-                    'name': self._contact.name,
-                    'reason': event.reason})
+                _("%(name)s cancelled the transfer (%(reason)s)")
+                % {"name": self._contact.name, "reason": event.reason}
+            )
             return
 
         if event.file_props.sid != self._file_props.sid:
@@ -252,29 +269,30 @@ class FileTransferJingleRow(BaseRow):
         elif isinstance(event, FileError):
             self._show_error(event.file_props)
         elif isinstance(event, FileHashError):
-            self._ui.action_stack.set_visible_child_name('hash-error')
-            self._ui.transfer_action.set_text(_('File Verification Failed'))
+            self._ui.action_stack.set_visible_child_name("hash-error")
+            self._ui.transfer_action.set_text(_("File Verification Failed"))
         elif isinstance(event, FileRequestError | FileSendError):
-            self._ui.action_stack.set_visible_child_name('error')
-            self._ui.transfer_action.set_text(_('File Transfer Cancelled'))
-            error_text = _('Connection with %s could not be '
-                           'established.') % self._contact.name
+            self._ui.action_stack.set_visible_child_name("error")
+            self._ui.transfer_action.set_text(_("File Transfer Cancelled"))
+            error_text = (
+                _("Connection with %s could not be established.") % self._contact.name
+            )
             if event.error_msg:
-                error_text = f'{error_text} ({event.error_msg})'
+                error_text = f"{error_text} ({event.error_msg})"
             self._ui.error_label.set_text(error_text)
 
         elif isinstance(event, FileProgress):
             self._update_progress(event.file_props)
 
     def _update_progress(self, file_props: FileProp) -> None:
-        self._ui.action_stack.set_visible_child_name('progress')
-        self._ui.transfer_action.set_text(_('Transferring File…'))
+        self._ui.action_stack.set_visible_child_name("progress")
+        self._ui.transfer_action.set_text(_("Transferring File…"))
 
         time_now = time.time()
         full_size = file_props.size
         assert full_size is not None
 
-        if file_props.type_ == 's':
+        if file_props.type_ == "s":
             # We're sending a file
             if self._start_time == 0:
                 self._start_time = time_now
@@ -290,55 +308,55 @@ class FileTransferJingleRow(BaseRow):
         if full_size == 0:
             return
 
-        bytes_sec = int(
-            round(transferred_size / (time_now - self._start_time), 1))
-        speed = f'{GLib.format_size_full(bytes_sec, self._units)}/s'
-        self._ui.progress_label.set_tooltip_text(_('Speed: %s') % speed)
+        bytes_sec = int(round(transferred_size / (time_now - self._start_time), 1))
+        speed = f"{GLib.format_size_full(bytes_sec, self._units)}/s"
+        self._ui.progress_label.set_tooltip_text(_("Speed: %s") % speed)
 
         if bytes_sec == 0:
-            eta = '∞'
+            eta = "∞"
         else:
-            eta = format_eta(round(
-                (full_size - transferred_size) / bytes_sec))
+            eta = format_eta(round((full_size - transferred_size) / bytes_sec))
 
         progress = float(transferred_size) / full_size
         self._ui.progress_label.set_text(
-            _('%(progress)s %% (%(time)s remaining)') % {
-                'progress': round(progress * 100),
-                'time': eta})
+            _("%(progress)s %% (%(time)s remaining)")
+            % {"progress": round(progress * 100), "time": eta}
+        )
 
         self._ui.progress_bar.set_fraction(progress)
 
     def _show_error(self, file_props: FileProp) -> None:
-        self._ui.action_stack.set_visible_child_name('error')
-        self._ui.transfer_action.set_text(_('File Transfer Stopped'))
+        self._ui.action_stack.set_visible_child_name("error")
+        self._ui.transfer_action.set_text(_("File Transfer Stopped"))
         if file_props.error == -1:
             self._ui.error_label.set_text(
-                _('%s stopped the transfer') % self._contact.name)
+                _("%s stopped the transfer") % self._contact.name
+            )
         elif file_props.error == -6:
-            self._ui.error_label.set_text(_('Error opening file'))
+            self._ui.error_label.set_text(_("Error opening file"))
         elif file_props.error == -12:
-            self._ui.error_label.set_text(_('SSL certificate error'))
+            self._ui.error_label.set_text(_("SSL certificate error"))
         else:
-            self._ui.error_label.set_text(_('An error occurred'))
+            self._ui.error_label.set_text(_("An error occurred"))
 
     def _show_completed(self) -> None:
-        self._ui.action_stack.set_visible_child_name('complete')
-        self._ui.transfer_action.set_text(_('File Transfer Completed'))
+        self._ui.action_stack.set_visible_child_name("complete")
+        self._ui.transfer_action.set_text(_("File Transfer Completed"))
 
     def _on_accept_file_request(self, _button: Gtk.Button) -> None:
-        app.interface.instances['file_transfers'].on_file_request_accepted(
-            self._account, self._contact, self._file_props)
-        self._start_time = time.time()
+        pass
+        # app.interface.instances["file_transfers"].on_file_request_accepted(
+        #     self._account, self._contact, self._file_props
+        # )
+        # self._start_time = time.time()
 
     def _on_reject_file_request(self, _button: Gtk.Button) -> None:
         assert self._file_props is not None
-        self._client.get_module('Bytestream').send_file_rejection(
-            self._file_props)
+        self._client.get_module("Bytestream").send_file_rejection(self._file_props)
         assert self._file_props is not None
         self._file_props.stopped = True
-        self._ui.action_stack.set_visible_child_name('rejected')
-        self._ui.transfer_action.set_text(_('File Transfer Cancelled'))
+        self._ui.action_stack.set_visible_child_name("rejected")
+        self._ui.transfer_action.set_text(_("File Transfer Cancelled"))
 
     def _on_open_file(self, _button: Gtk.Button) -> None:
         assert self._file_props is not None
@@ -351,18 +369,19 @@ class FileTransferJingleRow(BaseRow):
         show_in_folder(Path(self._file_props.file_name))
 
     def _on_bad_hash_retry(self, _button: Gtk.Button) -> None:
-        app.interface.instances['file_transfers'].show_hash_error(
-            self._contact.jid,
-            self._file_props,
-            self._account)
+        pass
+        # app.interface.instances["file_transfers"].show_hash_error(
+        #     self._contact.jid, self._file_props, self._account
+        # )
 
     def _on_cancel_transfer(self, _button: Gtk.Button) -> None:
-        app.interface.instances['file_transfers'].cancel_transfer(
-            self._file_props)
+        pass
+        # app.interface.instances["file_transfers"].cancel_transfer(self._file_props)
 
     def _on_show_transfers(self, _button: Gtk.Button) -> None:
-        file_transfers = app.interface.instances['file_transfers']
-        if file_transfers.window.get_property('visible'):
-            file_transfers.window.present()
-        else:
-            file_transfers.window.show_all()
+        pass
+        # file_transfers = app.interface.instances["file_transfers"]
+        # if file_transfers.window.get_property("visible"):
+        #     file_transfers.window.present()
+        # else:
+        #     file_transfers.window.show_all()
